@@ -2,6 +2,7 @@ import json
 import logging
 import aiohttp
 from expiringdict import ExpiringDict
+from datetime import datetime, timedelta
 
 from .const import (
     DEFAULT_DATA_CACHE_SECONDS,
@@ -60,15 +61,25 @@ class LinkStation:
         self._session = session
         self._api = None
         self._cache = ExpiringDict(max_len=10, max_age_seconds=cache_age)
+        self._authentication_expire_at = None
+
+    def _authentication_required(self) -> bool:
+        if (self._authentication_expire_at is None):
+            return True
+
+        if (self._authentication_expire_at < datetime.now):
+            return True
+
+        return False
+
+
 
     async def get_data_with_cache_async(self):
-        data = None
+        data = self._cache.get(LINKSTATION_API_REPONSE_DATA_ELEMENT)
 
-        if self._cache.get(LINKSTATION_API_REPONSE_DATA_ELEMENT) is not None:
-            data = self._cache.get(LINKSTATION_API_REPONSE_DATA_ELEMENT)
-        else:
+        if data is None:
             _LOGGER.debug("Data key{%s} missing from cache", LINKSTATION_API_REPONSE_DATA_ELEMENT)
-            data = await self.get_disks_info_async()
+            data = await self.get_disks_info_async()            
 
         return data
 
@@ -94,6 +105,7 @@ class LinkStation:
                 self._sid = self._get_user_sid(authData)
                 self._pagemode = self._get_pagemode(authData)
                 self._cookies = self._create_authentication_cookie()
+                self._authentication_expire_at = datetime.now() + timedelta(minute=10)
             else:
                 _LOGGER.error("Authentication failed")
 
@@ -122,23 +134,36 @@ class LinkStation:
 
     async def _get_settings_info(self):
 
-        if self._session == None:
+        if self._session == None or self._session.closed or self._authentication_required:
             await self.connect_async()
 
         params = {
             LINKSTATION_API_FUNCTION_PARAM_NAME: LINKSTATION_API_GETSETTINGS_FUNC_NAME
         }
 
-        async with self._session.get(
-            self._api, params=params, cookies=self._cookies
-        ) as settingresp:
-            settingInfo = json.loads(await settingresp.text())
+        try:
+            async with self._session.get(
+                self._api, params=params, cookies=self._cookies
+            ) as settingresp:
+                settingInfo = json.loads(await settingresp.text())
 
-            if self._is_success(settingInfo):
-                _LOGGER.debug(await settingresp.text())
-                self._cache['settingInfo'] = settingInfo
-                return settingInfo
-            return None
+                if self._is_success(settingInfo):
+                    _LOGGER.debug(await settingresp.text())
+                    self._cache['settingInfo'] = settingInfo
+                    return settingInfo
+                return None
+
+            # here, the async with context for the response ends, and the response is
+            # released.
+        except aiohttp.ClientConnectionError:
+            # something went wrong with the exception, decide on what to do next
+            _LOGGER.error("Oops, the connection was dropped before we finished", exc_info=True)
+        except aiohttp.ClientError as client_error:
+            # something went wrong in general. Not a connection error, that was handled
+            # above.
+            _LOGGER.error("Oops, something else went wrong with the request %s", client_error.with_traceback, exc_info=True)
+
+        
 
     async def get_spaces_info_desc_async(self):
         return await self._get_settingsinfo_field_async("r_storage")
@@ -168,7 +193,7 @@ class LinkStation:
                 return data["value"]
 
     async def restart_async(self):
-        if self._session == None:
+        if self._session == None or self._session.closed or self._authentication_required:
             await self.connect_async()
 
         formdata = aiohttp.FormData()
@@ -188,13 +213,13 @@ class LinkStation:
                 _LOGGER.info("LinkStation restarting ... ")
 
     async def get_disks_info_with_cache_async(self):
-        if self._cache[LINKSTATION_API_REPONSE_DATA_ELEMENT]:
-            return self._cache[LINKSTATION_API_REPONSE_DATA_ELEMENT]
+        if self._cache.get(LINKSTATION_API_REPONSE_DATA_ELEMENT):
+            return self._cache.get(LINKSTATION_API_REPONSE_DATA_ELEMENT)
 
         return await self.get_disks_info_async()
 
     async def get_disks_info_async(self):
-        if self._session == None or self._session.closed:
+        if self._session == None or self._session.closed or self._authentication_required:
             await self.connect_async()
 
         formdata = aiohttp.FormData()
@@ -202,17 +227,33 @@ class LinkStation:
             LINKSTATION_API_FUNCTION_PARAM_NAME, LINKSTATION_API_GETALLDISK_FUNC_NAME
         )
 
-        async with self._session.post(
+        try:
+            async with self._session.post(
             self._api, data=formdata, cookies=self._cookies
-        ) as getdisksresp:
-            getdisksinfo = json.loads(await getdisksresp.text())
-            _LOGGER.debug(await getdisksresp.text())
+            ) as getdisksresp:
+                response_str = await getdisksresp.text()
+                getdisksinfo = json.loads(response_str)
+                _LOGGER.debug(response_str)
 
-            if self._is_success(getdisksinfo):
-                self._cache[LINKSTATION_API_REPONSE_DATA_ELEMENT] = getdisksinfo[LINKSTATION_API_REPONSE_DATA_ELEMENT]
-                return getdisksinfo[LINKSTATION_API_REPONSE_DATA_ELEMENT]
-            else:
-                return None
+                if self._is_success(getdisksinfo):
+                    self._cache[LINKSTATION_API_REPONSE_DATA_ELEMENT] = getdisksinfo[LINKSTATION_API_REPONSE_DATA_ELEMENT]
+                    return getdisksinfo[LINKSTATION_API_REPONSE_DATA_ELEMENT]
+                else:
+                    _LOGGER.error("Server return unsuccess response %s %s", getdisksresp.reason, response_str)
+                    return None
+            # here, the async with context for the response ends, and the response is
+            # released.
+        except aiohttp.ClientConnectionError:
+            # something went wrong with the exception, decide on what to do next
+            _LOGGER.error("Oops, the connection was dropped before we finished", exc_info=True)
+        except aiohttp.ClientError as client_error:
+            # something went wrong in general. Not a connection error, that was handled
+            # above.
+            _LOGGER.error("Oops, something else went wrong with the request %s", client_error.with_traceback, exc_info=True)
+        
+        return None
+
+        
 
     async def get_all_disks_async(self):
 
@@ -240,6 +281,9 @@ class LinkStation:
     async def get_disk_status_async(self, diskName):
         diskInfo = await self.get_data_with_cache_async()
 
+        if diskInfo is None :
+            return None
+
         for data in diskInfo:
             if data[LINKSTATION_API_REPONSE_DATA_DISK_ELEMENT] == diskName:
                 return data[LINKSTATION_API_REPONSE_DATA_DISK_STATUS]
@@ -289,6 +333,9 @@ class LinkStation:
         """Get disk capacity, data return in GB"""
         diskInfo = self._cache.get(LINKSTATION_API_REPONSE_DATA_ELEMENT)
 
+        if diskInfo is None:
+            return None
+
         for data in diskInfo:
             if data[LINKSTATION_API_REPONSE_DATA_DISK_ELEMENT] == diskName:
                 return self._format_disk_space(
@@ -313,6 +360,9 @@ class LinkStation:
         """Get disk spaces used, data return in GB"""
         diskInfo = self._cache.get(LINKSTATION_API_REPONSE_DATA_ELEMENT)
 
+        if diskInfo is None:
+            return None
+
         for data in diskInfo:
             if data[LINKSTATION_API_REPONSE_DATA_DISK_ELEMENT] == diskName:
                 return self._format_disk_space(
@@ -345,6 +395,8 @@ class LinkStation:
         """Get disk space used, data return in percentage"""
         diskInfo = self._cache.get(LINKSTATION_API_REPONSE_DATA_ELEMENT)
 
+        if diskInfo is None:
+            return None
         for data in diskInfo:
             if data[LINKSTATION_API_REPONSE_DATA_DISK_ELEMENT] == diskName:
                 return self._format_disk_pct(
@@ -371,6 +423,8 @@ class LinkStation:
         """Get disk space used, data return in percentage"""
         diskInfo = diskInfo = self._cache.get(LINKSTATION_API_REPONSE_DATA_ELEMENT)
 
+        if diskInfo is None:
+            return None
         for data in diskInfo:
             if data[LINKSTATION_API_REPONSE_DATA_DISK_ELEMENT] == diskName:
                 return self._format_disk_space(
